/*!
 * \file        buscaseq.c
 * \brief       Função da busca sequencial
 *
 * \date        01-05-2012 16:35
 * \author      Alessandro Elias, ae11@inf.ufpr.br
 * \author      Ruanito Diego Santos, rds@inf.ufpr.br
 */

/*!
 * Busca squencial.
 *
 * \param       [in] ele - elemento sendo procurado.
 * \param       [in, out] vec - vetor de inteiros. Somente o elemento 0
 *              será alterado.
 * \param       [in] size - número de elementos no vetor.
 * \param       [out] ncomparison - número de comparações feitas,
 *              até encontrar ele, ou size se não for encontrado.
 *
 *  \return     -1 'ele' não foi encontrado, ou o índice de 'ele'.
 *              ncomparasion retorna número de comparações.
 *
 *  \remarks    Caso o ponteiro para ncomparasion for inválido, SEGV ocorrerá.
 */
int busca_seq(int ele, int vec[], int size, int *ncomparison)
{
    int i;

    vec[0] = ele;
    for( i=size-1; i >= 0 && ele != vec[i]; i-- )
        ;

    *ncomparison = size - i;    /* the ith element is the total elements minus position of element where it has been found. */
    return( i ? i : -1);        /* -1 has been not found. */
}/* busca_seq */
