/*!
 * \file        sortui.h
 * \brief       Protótipos e esqueletos de dados para o módulo sortui.c.
 *
 * \date        02-05-2012 16:36:45
 * \version     0.0.1
 * \author      Alessandro Elias, ae11@inf.ufpr.br
 * \author      Ruanito Diego Santos, rds@inf.ufpr.br
 */

#ifndef __SORTUI_H__
#define __SORTUI_H__      1

#include "mycommdef.h"

/*! Define número padrão de colunas. Em caso de erro este será o tamanho adotado. */
#define DEFAULT_COLS        80
/*! Define número de linhas padrão. Em caso de erro este será o tamanho adotado. */
#define DEFAULT_LINES       25

/*! Define tamanho do vetor. Posição 0 utilizada
 * para busca com sentinela. Somente 512 posições
 * são utilizadas.
 */
#define SIZE_OF_ARRAY       513

/*! Elemento mínimo gerado aleatóriamente.  */
#define MIN_ELEMENT         0
/*! Elemento máximo gerado aleatóriamente. */
#define MAX_ELEMENT         1000
/*! Número de casos no qual serão calculados médias e devio padrão.  */
#define MAX_AVERAGE         1000

/*! \brief Opções do menu. */
enum eOptMenu {
    oGenerateArray = 1,         /*!< Gera vetor aleatóriamente. */
    oSearch,                    /*!< Executa pesquisa (sequencial e binária). */
    oSort,                      /*!< Ordena com os algoritmos select e quicksort. */
    oSS1000,                    /*!< Faz pequiza e ordenação em 1000 casos, cálcula média e devio padrão e gera relatório. */
    oPrintUnsorted = 51,        /*!< Imprime na saída padrão o vetor desordenado.*/
    oPrintSorted,               /*!< Imprime na saída padrão o vetor ordenado. */
    oExit = 6,                  /*!< Sai do programa. */
    oUnknown,                   /*!< Opção inválida. */
    oNone                       /*!< Não faça nada */
};
 
/*! \brief Esqueleto dos dados do relatório de ordenação. */
struct SORT_REPORT {
    int comparasions;       /*!< Número de comparações. */
    int permutations;       /*!< Número de permutações. */
    float average;          /*!< Média das comparações. */
    float std_deviation;    /*!< Desvio padrão das comparações. */
};

/*! \brief Esqueleto dos dados do relatório de pesquisa. */
struct SEARCH_REPORT {
    int seq_index;          /*!< Índice do elemento encontrado no vetor pela busca sequencial. -1 não encontrado. */
    int bin_index;          /*!< Índice do elemento encontrado no vetor pela busca binária. -1 não encontrado. */
    int seq_comp;           /*!< Número de comparações feitas pela busca sequencial. */
    int bin_comp;           /*!< Número de comparações feitas pela busca binária. */
    float seq_average;      /*!< Média das comparações da busca sequencial. */
    float seq_std_deviation;    /*!< Desvio padrão da busca sequencial. */
    float bin_average;          /*!< Média das comparações da busca binária. */
    float bin_std_deviation;    /*!< Desvio padrão da busca binária. */
};

void PrintMenu(void);
void Greeting(void);
int  read_input(char*, int);
enum eOptMenu ParseOption(const char*, enum eOptMenu);
bool MoreInput(void);
void GenerateRndArray(int[], int);
void PrintArray(int[], int);
int  get_element(void);
void ReportSearch(int, bool, bool, const struct SEARCH_REPORT*);
void ReportSort(const struct SORT_REPORT*, const struct SORT_REPORT*);
void ReportAverage(const struct SEARCH_REPORT*,
            const struct SORT_REPORT*, const struct SORT_REPORT*);
void get_AverageAndStdDeviation(struct SEARCH_REPORT *s_rpt,
                                struct SORT_REPORT *ss_rpt,
                                struct SORT_REPORT *qks_rpt,
                                int nsize);

#endif /*__SORTUI_H__*/
